from dataclasses import dataclass
import pandas as pd
import ast


@dataclass
class prefilters:
    # col names of dataframe
    reason_codes: list
    col_user_cnt: str = "userid"
    col_reason_codes: str = "reason_codes"
    col_underwriting_decision: str = "underwriting_decision"
    reason_codes_len: int = 20


    def _hit(self, x, prefilter_code):
        """
        function to check if a reason code exists in the whole reason code string
        """
        if prefilter_code in x[self.col_reason_codes]:
            return True
        else:
            return False

    def _first_hit(self, x, prefilter_code):
        """
        function to check if a reason code exists in the first position of the reason code string
        """
        reason_codes = ast.literal_eval(x[self.col_reason_codes]) #convert string to list
        first_reason = reason_codes[0]

        if prefilter_code == first_reason:
            return True
        else:
            return False

    def _only_hit(self, x, prefilter_code):
        """
        function to check if a reason code is the only reaon code exist in the reason code string
        """

        reason_codes_str = []
        reason_codes_str.append(prefilter_code)
        for i in range(self.reason_codes_len-1):
            reason_codes_str.append('')

        if x[self.col_reason_codes] == str(reason_codes_str):
            return True
        else:
            return False

    def generate_hit_cols(self, df):
        for code in self.reason_codes:
            df[f'{code}_hit'] = df.apply(self._hit, prefilter_code=code, axis=1)

        for code in self.reason_codes:
            df[f'{code}_first_hit'] = df.apply(self._first_hit, prefilter_code=code, axis=1)

        for code in self.reason_codes:
            df[f'{code}_only_hit'] = df.apply(self._only_hit, prefilter_code=code, axis=1)

        return df

    def rejection_rate(self, x, prefilter_code):
        d= {}
        # rejection rate out of total activation users
        d[f'{prefilter_code}_only_hit_rate'] = x[f'{prefilter_code}_only_hit'].sum()/x[self.col_user_cnt].count()
        d[f'{prefilter_code}_hit_rate'] = x[f'{prefilter_code}_hit'].sum()/x[self.col_user_cnt].count()
        d[f'{prefilter_code}_first_hit_rate'] = x[f'{prefilter_code}_first_hit'].sum()/x[self.col_user_cnt].count()

        # count
        d[f'{prefilter_code}_only_hit_cnt'] = x[f'{prefilter_code}_only_hit'].sum()
        d[f'{prefilter_code}_hit_cnt'] = x[f'{prefilter_code}_hit'].sum()
        d[f'{prefilter_code}_first_hit_cnt'] = x[f'{prefilter_code}_first_hit'].sum()

        # rejection rate out of rejected users
        d[f'{prefilter_code}_only_hit_rate_of_reject'] = x[f'{prefilter_code}_only_hit'].sum()/x[x[self.col_underwriting_decision]=='Reject'][self.col_user_cnt].count()
        d[f'{prefilter_code}_hit_rate_of_reject'] = x[f'{prefilter_code}_hit'].sum()/x[x[self.col_underwriting_decision]=='Reject'][self.col_user_cnt].count()
        d[f'{prefilter_code}_first_hit_rate_of_reject'] = x[f'{prefilter_code}_first_hit'].sum()/x[x[self.col_underwriting_decision]=='Reject'][self.col_user_cnt].count()

        return pd.Series(d, index=[f'{prefilter_code}_only_hit_rate',f'{prefilter_code}_hit_rate',f'{prefilter_code}_first_hit_rate',
            f'{prefilter_code}_only_hit_cnt',f'{prefilter_code}_hit_cnt',f'{prefilter_code}_first_hit_cnt',
            f'{prefilter_code}_only_hit_rate_of_reject',f'{prefilter_code}_hit_rate_of_reject',f'{prefilter_code}_first_hit_rate_of_reject',
            ])

@dataclass
class prefilters_v2:
    reason_codes: list
    col_reason_codes: str = "reason_codes"
    col_reason_code_01: str = "reason_code_01"
    col_reason_code_02: str = "reason_code_02"
    col_user_cnt: str = "userid"
    col_underwriting_decision: str = "underwriting_decision"
    reason_codes_len: int = 20

    def _hit(self, x, prefilter_code):
        """
        function to check if a reason code exists in the whole reason code string
        """
        if prefilter_code in x[self.col_reason_codes]:
            return True
        else:
            return False

    def _first_hit(self, x, prefilter_code):
        """
        function to check if a reason code exists in the first position of the reason code string
        """

        if prefilter_code == x[self.col_reason_code_01]:
            return True
        else:
            return False

    def _only_hit(self, x, prefilter_code):
        """
        function to check if a reason code is the only reaon code exist in the reason code string
        """

        if prefilter_code == x[self.col_reason_code_01] and x[self.col_reason_code_02]=="":
            return True
        else:
            return False

    def _generate_split_col_names(self):
        """
        Generate column names to store the split reason code string
        """
        split_col_names = []
        for i in range(1,self.reason_codes_len+1):
            code = str(i).zfill(2)
            reason_code_name = f"reason_code_{code}"
            split_col_names.append(reason_code_name)

        return split_col_names


    def decompose_reason_codes(self, df):
        """
        If reason code is still in one col, decompose into multiple cols
        """
        split_col_names = self._generate_split_col_names()

        df[self.col_reason_codes] = df[self.col_reason_codes].apply(lambda x: ast.literal_eval(str(x))) #convert string list col to list col
        df[split_col_names] = pd.DataFrame(df[self.col_reason_codes].tolist(), index=df.index)
        return df 

    def generate_hit_cols(self, df):
        for code in self.reason_codes:
            df[f'{code}_hit'] = df.apply(self._hit, prefilter_code=code, axis=1)

        for code in self.reason_codes:
            df[f'{code}_first_hit'] = df.apply(self._first_hit, prefilter_code=code, axis=1)

        for code in self.reason_codes:
            df[f'{code}_only_hit'] = df.apply(self._only_hit, prefilter_code=code, axis=1)

        return df

    def rejection_rate(self, x, prefilter_code):
        d= {}
        # rejection rate out of total activation users
        d[f'{prefilter_code}_only_hit_rate'] = x[f'{prefilter_code}_only_hit'].sum()/x[self.col_user_cnt].count()
        d[f'{prefilter_code}_hit_rate'] = x[f'{prefilter_code}_hit'].sum()/x[self.col_user_cnt].count()
        d[f'{prefilter_code}_first_hit_rate'] = x[f'{prefilter_code}_first_hit'].sum()/x[self.col_user_cnt].count()

        # count
        d[f'{prefilter_code}_only_hit_cnt'] = x[f'{prefilter_code}_only_hit'].sum()
        d[f'{prefilter_code}_hit_cnt'] = x[f'{prefilter_code}_hit'].sum()
        d[f'{prefilter_code}_first_hit_cnt'] = x[f'{prefilter_code}_first_hit'].sum()

        # rejection rate out of rejected users
        d[f'{prefilter_code}_only_hit_rate_of_reject'] = x[f'{prefilter_code}_only_hit'].sum()/x[x[self.col_underwriting_decision]=='Reject'][self.col_user_cnt].count()
        d[f'{prefilter_code}_hit_rate_of_reject'] = x[f'{prefilter_code}_hit'].sum()/x[x[self.col_underwriting_decision]=='Reject'][self.col_user_cnt].count()
        d[f'{prefilter_code}_first_hit_rate_of_reject'] = x[f'{prefilter_code}_first_hit'].sum()/x[x[self.col_underwriting_decision]=='Reject'][self.col_user_cnt].count()

        return pd.Series(d, index=[f'{prefilter_code}_only_hit_rate',f'{prefilter_code}_hit_rate',f'{prefilter_code}_first_hit_rate',
            f'{prefilter_code}_only_hit_cnt',f'{prefilter_code}_hit_cnt',f'{prefilter_code}_first_hit_cnt',
            f'{prefilter_code}_only_hit_rate_of_reject',f'{prefilter_code}_hit_rate_of_reject',f'{prefilter_code}_first_hit_rate_of_reject',
            ])


from pyhive import presto
import pandas as pd
import numpy as np

class getdata:
    def get_query_from_file(self, filepath):
        fd = open(filepath, 'r')
        query = fd.read()
        return query

    def run_shopee_presto(self, query, conn):
        """
        get shopee data from presto using jdbc
        """
        cur = conn.cursor()
        cur.execute(query)
        rows = cur.fetchall()
        desc = cur.description
        col_name = [x[0] for x in desc]

        df = pd.DataFrame(rows, columns=col_name)

        return df


from dataclasses import dataclass
import pandas as pd


@dataclass
class dpd:
    # col names of dataframe
    col_fpdxp_amt: str = None #flexible fpd
    col_is_fpdx: str = None #flexible fpd

    col_fpd0p_amt: str = "user_fpd0_principal"
    col_fpd7p_amt: str = "user_fpd7_principal"
    col_fpd15p_amt: str = "user_fpd15_principal"
    col_fpd30p_amt: str = "user_fpd30_principal"
    col_user_principal: str = "user_principal"
    col_is_fpd0: str = "is_fpd0"
    col_is_fpd7: str = "is_fpd7"
    col_is_fpd15: str = "is_fpd15"
    col_is_fpd30: str = "is_fpd30"
    col_user_cnt: str = "userid"


    def dpd_basic(self, x):
        """
        Calculate fpd amount level and user level statistics from df
        expect to group by before applying the function
        example: df.groupby(['user_type']).apply(dpd_basic)
        """
        d = {}
        d['fpdxp_amt'] = (x[self.col_fpdxp_amt].sum()/x[self.col_user_principal].sum())*100
        d['fpd0p_amt'] = (x[self.col_fpd0p_amt].sum()/x[self.col_user_principal].sum())*100
        d['fpd7p_amt'] = (x[self.col_fpd7p_amt].sum()/x[self.col_user_principal].sum())*100
        d['fpd15p_amt'] = (x[self.col_fpd15p_amt].sum()/x[self.col_user_principal].sum())*100
        d['fpd30p_amt'] = (x[self.col_fpd30p_amt].sum()/x[self.col_user_principal].sum())*100 

        d['fpdxp_user'] = (x[self.col_is_fpdx].sum()/x[self.col_user_cnt].count())*100
        d['fpd0p_user'] = (x[self.col_is_fpd0].sum()/x[self.col_user_cnt].count())*100
        d['fpd7p_user'] = (x[self.col_is_fpd7].sum()/x[self.col_user_cnt].count())*100
        d['fpd15p_user'] = (x[self.col_is_fpd15].sum()/x[self.col_user_cnt].count())*100
        d['fpd30p_user'] = (x[self.col_is_fpd30].sum()/x[self.col_user_cnt].count())*100
        d['user_cnt'] = x[self.col_user_cnt].count()
        return pd.Series(d, index=['fpd0p_amt', 'fpd7p_amt','fpd15p_amt','fpd30p_amt','fpd0p_user','fpd7p_user','fpd15p_user','fpd30p_user','user_cnt'])

    def dpd_checking(self, x):
        """
        Calculate fpd amount level and user level statistics from df
        expect to group by before applying the function
        example: df.groupby(['user_type']).apply(dpd_basic)
        """
        d = {}
        d['fpdxp_principal'] = x[self.col_fpdxp_amt].sum()
        d['fpd0p_principal'] = x[self.col_fpd0p_amt].sum()
        d['fpd7p_principal'] = x[self.col_fpd7p_amt].sum()
        d['fpd15p_principal'] = x[self.col_fpd15p_amt].sum()
        d['fpd30p_principal'] = x[self.col_fpd30p_amt].sum()

        d['fpdxp_user_cnt'] = x[self.col_is_fpdx].sum()
        d['fpd0p_user_cnt'] = x[self.col_is_fpd0].sum()
        d['fpd7p_user_cnt'] = x[self.col_is_fpd7].sum()
        d['fpd15p_user_cnt'] = x[self.col_is_fpd15].sum()
        d['fpd30p_user_cnt'] = x[self.col_is_fpd30].sum()
        d['user_cnt'] = x[self.col_user_cnt].count()
        d['user_principal'] = x[self.col_user_principal].sum()
        return pd.Series(d, index=['fpd0p_principal', 'fpd7p_principal','fpd15p_principal','fpd30p_principal','fpd0p_user_cnt','fpd7p_user_cnt','fpd15p_user_cnt','fpd30p_user_cnt','user_cnt', 'user_principal'])





from setuptools import setup, find_packages

setup(
   name='johnshopeetools',
   version='0.1.14',
   author='John Chan | making difficult live easier',
   author_email='john.chanky@seamoney.com',
   packages=find_packages(),
   description='A toolbox for credit policy analysis',
   install_requires=[
       "pandas",
   ],
)